import 'package:bomber_barbell/basicUI.dart';
import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:fl_chart/fl_chart.dart';
import 'dart:math';



/*
* Fix out of range issues to use animation
* */


class DataDetail extends StatefulWidget {
  static const routeName = '/dataDetail';
  @override
  _DataDetailState createState() => _DataDetailState();
}

class _DataDetailState extends State<DataDetail> {
  int _timeType = 0;
  final List<String> _timeTypeLabels = ['days', 'weeks', 'months', 'years'];

  final List<String> _daysLabels = ['S', 'M', 'T', 'W', 'T', 'F', 'S'];
  final List<int> _daysData = [3, 6, 3, 1, 5, 4, 7];

  final List<String> _weeksLabels = ['W1', 'W2', 'W3', 'W4', 'W5'];
  final List<int> _weeksData = [6, 3, 4, 2, 1];

  final List<String> _monthsLabels = ['J', 'F', 'M', 'A', 'M', 'J', 'J', 'A', 'S', 'O', 'N', 'D'];
  final List<int> _monthsData = [14, 21, 16, 18, 11, 12, 17, 25, 22, 20, 19, 22];

  final List<String> _yearsLabels = ['Y1', 'Y2', 'Y3', 'Y4', 'Y5'];
  final List<int> _yearsData = [140, 100, 125, 117, 100];



  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        backgroundColor: Color.fromARGB(255, 10, 71, 125),
        elevation: 0,
        title: Text(
          'Muscle Group',
          style: TextStyle(
              color: Colors.white,
              fontFamily: 'Rubik',
              fontWeight: FontWeight.bold
          ),
          textAlign: TextAlign.center,
        ),
      ),
      body: ListView(
        padding: EdgeInsets.symmetric(horizontal: 30, vertical: 15),
        children: [
          Padding(
            padding: const EdgeInsets.symmetric(vertical: 15),
            child: BasicCard(
              Column(
                crossAxisAlignment: CrossAxisAlignment.center,
                mainAxisAlignment: MainAxisAlignment.center,
                children: [
                  Padding(
                    padding: const EdgeInsets.symmetric(horizontal: 25),
                    child: Row(
                      crossAxisAlignment: CrossAxisAlignment.center,
                      children: List.generate(_timeTypeLabels.length, (index) {
                        final bool _selected = (_timeType == index);
                        return Expanded(
                          flex: 1,
                          child: RawMaterialButton(
                            elevation: 0,
                            constraints: BoxConstraints(
                              minWidth: 60,
                              maxWidth: 76,
                              minHeight: 33,
                              maxHeight: 33
                            ),
                            child: Text(
                              _timeTypeLabels[index],
                              style: TextStyle(
                                  fontWeight: FontWeight.w300,
                                  color: Colors.black,
                                  fontSize: 18,

                              ),
                              maxLines: 1,
                            ),
                            fillColor: _selected ? BasicColors.grey : Colors.white,
                            shape: RoundedRectangleBorder(
                              borderRadius: BorderRadius.horizontal(
                                  left: Radius.circular(index == 0 ? 8 : 0),
                                  right: Radius.circular(index == _timeTypeLabels.length-1 ? 8 : 0)
                              ),
                              side: BorderSide(
                                color: BasicColors.grey,
                              ),
                            ),
                            onPressed: () {
                              setState(() {
                                _timeType = index;
                              });
                            },
                          ),
                        );
                      })
                    ),
                  ),
                  Padding(
                    padding: const EdgeInsets.all(8.0),
                    child: Graph(
                      timeType: _timeType,
                      daysLabels: _daysLabels,
                      daysData: _daysData,
                      weeksLabels: _weeksLabels,
                      weeksData: _weeksData,
                      monthsLabels: _monthsLabels,
                      monthsData: _monthsData,
                      yearsLabels: _yearsLabels,
                      yearsData: _yearsData,
                    ),
                  ),
                  Row(
                    mainAxisAlignment: MainAxisAlignment.center,
                    children: [
                      Text(
                        'compare with',
                        style: TextStyle(
                          fontSize: 16,
                        ),
                        textAlign: TextAlign.center,
                      ),
                      Padding(
                        padding: const EdgeInsets.only(left: 8),
                        child: BasicButton(
                          title: 'muscle group',
                          height: 26,
                          width: 112,
                          fontSize: 14.4,
                          onPressed: () => print('compare'),
                        ),
                      )
                    ],
                  )
                ],
              ),
              padding: EdgeInsets.only(top: 19, bottom: 19,),
            ),
          ),
          Padding(
            padding: const EdgeInsets.symmetric(vertical: 15),
            child: BasicCard(
              Column(
                crossAxisAlignment: CrossAxisAlignment.center,
                children: [
                  Row(
                    children: [
                      Text(
                        'Movement',
                        style: TextStyle(
                          fontFamily: 'Rubik',
                          fontWeight: FontWeight.w500,
                          fontSize: 21
                        ),
                      ),
                      Spacer()
                    ],
                  ),
                  Padding(
                    padding: const EdgeInsets.only(top: 20),
                    child: Container(
                      height: 1,
                      color: BasicColors.grey,
                    ),
                  ),
                  Padding(
                    padding: const EdgeInsets.all(8.0),
                    child: Text(
                      '3 rep max: 50 lbs',
                      style: TextStyle(
                        fontWeight: FontWeight.w300,
                        fontSize: 18
                      ),
                    ),
                  ),
                  Container(
                    height: 1,
                    color: BasicColors.grey,
                  ),
                  Padding(
                    padding: const EdgeInsets.all(8.0),
                    child: Text(
                      '3 rep max: 50 lbs',
                      style: TextStyle(
                          fontWeight: FontWeight.w300,
                          fontSize: 18
                      ),
                    ),
                  ),
                  Container(
                    height: 1,
                    color: BasicColors.grey,
                  ),
                  Padding(
                    padding: const EdgeInsets.only(top: 16.0),
                    child: BasicButton(
                      title: 'view full data',
                      onPressed: () => print('view data'),
                    ),
                  )
                ],
              ),
              padding: const EdgeInsets.fromLTRB(19, 19, 19, 8),
            ),
          )
        ],
      ),
    );
  }
}

class Graph extends StatefulWidget {
  final int timeType;

  final List<String> daysLabels;
  final List<int> daysData;

  final List<String> weeksLabels;
  final List<int> weeksData;

  final List<String> monthsLabels;
  final List<int> monthsData;

  final List<String> yearsLabels;
  final List<int> yearsData;

  Graph({
    @required this.timeType,
    @required this.daysLabels,
    @required this.daysData,
    @required this.weeksLabels,
    @required this.weeksData,
    @required this.monthsLabels,
    @required this.monthsData,
    @required this.yearsLabels,
    @required this.yearsData
  });

  @override
  _GraphState createState() => _GraphState();
}

class _GraphState extends State<Graph> {
  final List<Color> gradientColors = [
    const Color.fromARGB(255, 11, 99, 223),
    const Color.fromARGB(69, 151, 207, 255),
    Colors.white.withOpacity(0.0)
  ];
  List<String> _labels;
  List<int> _data;

  void _setup() {
    switch (widget.timeType) {
      case 0:
        _labels = widget.daysLabels;
        _data = widget.daysData;
        break;
      case 1:
        _labels = widget.weeksLabels;
        _data = widget.weeksData;
        break;
      case 2:
        _labels = widget.monthsLabels;
        _data = widget.monthsData;
        break;
      case 3:
        _labels = widget.yearsLabels;
        _data = widget.yearsData;
        break;
      default:
        print('DATA DETAIL INIT ERROR');
        print('TIME TYPE: ${widget.timeType}');
    }
  }

  bool _shouldSpace(double value, int diff) {
    return (value % (diff > 40 ? 3:2) == 0);
  }

  @override
  Widget build(BuildContext context) {
    _setup();
    return Stack(
      children: <Widget>[
        AspectRatio(
          aspectRatio: 1.70,
          child: Padding(
            padding: const EdgeInsets.only(right: 12.0, left: 0.0, top: 24, bottom: 12),
            child: LineChart(
              mainData(),
              //swapAnimationDuration: Duration(seconds: 0),  // Quick fix
            ),
          ),
        ),
      ],
    );
  }

  LineChartData mainData() {
    final int diff = _data.reduce(max) - _data.reduce(min);
    return LineChartData(
      // clipData: FlClipData.all(),
      lineTouchData: LineTouchData(
        touchTooltipData: LineTouchTooltipData(
          fitInsideHorizontally: true,
          tooltipBgColor: Colors.white,
          getTooltipItems: (List<LineBarSpot> touchedBarSpots) {
            return touchedBarSpots.map((barSpot) {
              return LineTooltipItem(
                '${barSpot.y.toInt()}',
                TextStyle(
                  fontFamily: 'Jost*',
                  fontSize: 15,
                  color: Colors.black,
                ),
              );
            }).toList();
          }
        ),
        getTouchedSpotIndicator: (LineChartBarData barData, List<int> spotIndexes) {
          return spotIndexes.map((spotIndex) {
            return TouchedSpotIndicatorData(
              FlLine(
                color: const Color.fromARGB(255, 77, 77, 77),
                strokeWidth: 1,
                dashArray: [4,4],
              ),
              FlDotData(
                getDotPainter: (spot, percent, barData, index) {
                  return FlDotCirclePainter(
                    radius: 5.5,
                    color: gradientColors[0],
                    strokeWidth: 2,
                    strokeColor: Colors.white,
                  );
                },
              ),
            );
          }).toList();
        }
      ),
      gridData: FlGridData(
        show: true,
        getDrawingHorizontalLine: (value) {
          Color color = const Color.fromARGB(255, 98, 95, 161);
          if (diff > 7) {
            color = Colors.white.withOpacity(0);
            if (_shouldSpace(value, diff)) color = const Color.fromARGB(255, 98, 95, 161);
          }
          if (value == _data.reduce(max) + 1) color = Colors.white.withOpacity(0);
          return FlLine(
            color: color,
            strokeWidth: 1,
            dashArray: [4,4]
          );
        },
      ),
      titlesData: FlTitlesData(
        show: true,
        bottomTitles: SideTitles(
          showTitles: true,
          reservedSize: 14,
          textStyle:
          const TextStyle(
            color: Color.fromARGB(255, 181, 181, 181),
            fontWeight: FontWeight.w300,
            fontFamily: 'Jost*',
            fontSize: 13,
          ),
            getTitles: (value) {
              return _labels[value.toInt()] ?? '';
            },
        ),
        leftTitles: SideTitles(
          showTitles: true,
          textStyle: const TextStyle(
            color: Color.fromARGB(255, 181, 181, 181),
            fontWeight: FontWeight.w300,
            fontFamily: 'Jost*',
            fontSize: 16,
          ),
          getTitles: (value) {
            final int num = value.toInt();
            if (diff > 7) {
              if (_shouldSpace(value, diff)) return num.toString();
              return '';
            }
            return num.toString();
          },
          //reservedSize: 28,
          margin: 9
        ),
      ),
      borderData:
      FlBorderData(
        show: true,
        border: Border.symmetric(
          horizontal: BorderSide(
            color: const Color.fromARGB(255, 170, 170, 170),
            width: 1.2
          ),
        ),
      ),
      minX: 0,
      maxX: _data.length.toDouble()-1, //length of data set
      minY: _data.reduce(min).toDouble() - 1,  //set to lowest v
      maxY: _data.reduce(max).toDouble() + 1,  //set to highest v
      lineBarsData: [
        LineChartBarData(
          spots: [
            for (int i = 0; i < _data.length; i++)
              FlSpot(i.toDouble(), _data[i].toDouble())
          ],
          //FlSpot(2.6, 4),
          isCurved: true,
          colors: [
            gradientColors[0],
          ],
          barWidth: 2,
          isStrokeCapRound: true,
          dotData: FlDotData(
            show: false,
          ),
          belowBarData: BarAreaData(
            show: true,
            colors: gradientColors,
            gradientColorStops: [0, 0.5, 1.0],
            gradientFrom: const Offset(0, 0),
            gradientTo: const Offset(0, 1),
          ),
        ),
      ],
    );
  }
}
